clc;close all; clear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   1. Set Parameters

global r gamma alpha_n  eta beta kappa glower h phi rho sigma

%%%%%%%%%%%%%%%%%% Parameters taken from literature

r          = .06;                     %% risk-free interest rate
gamma = 2;                       %% risk aversion parameter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% Auxiliary Variables

gLR=1.023;                        %% long run growth
gupper=1.05;                     %% highest growth rate: arbitrarily set

%%%%%%%%%%%%%%%%%%% Parameters set to match moments

beta=0.968;                       %% discount rate
glower=0.95;                     %%  parameter in Psi function
eta=2;                               %%  quardratic functional form

investment=1-0.3*(1+r-gLR)/gLR-.776;                            %% investment is implied by the budget constraint in steady states
i_aux=investment-gLR+glower;                                         %% i_aux is the quardratic term in the Psi function
h=1+glower+i_aux/(gLR-glower)*((eta-1)*gLR+glower)...
    -(1+eta/(gLR-glower)*i_aux)/beta*gLR^gamma;             %% habit parameter, chosen to match Euler Equation for z
kappa=i_aux/(gLR-glower)^eta;                                         %% Average Growth Enhancing Expenditure

bLR=-0.3/gLR;                                                                  %% NFA=-30% of GDP
alpha_n=0.2;                                                                     %% Share of Asset Income
qLR=beta*gLR^(1-gamma)/(1-beta*gLR^(1-gamma))*alpha_n; %% Asset price in steady state
phi=(-bLR*gLR)/qLR;                                                             %% Collateral Constraint Parameter


%%%%%%%%%%%%%%%%%  Exogenous TFP process %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rho=.83;                             %% persistence parameter
sigma=0.04;                       %% volatility parameter


NA=5;                    %% # of shocks space
[logA, Prob]=rouwenhorst(rho,sigma,NA);  %% use Rouwenhorst's method to discretize

A=exp(logA)'; %% realization of shocks
state_Prob=Prob^1000; state_Prob=state_Prob(1,:);%% Long run Probability
EA=state_Prob*A; %% long run technology

%%%%%%%%%%%%%%%%%%%%%%%%%% Create State Space
%% Find the minimum level of bond in the state space %%%%%%%%%%%
gmin=glower;
qmin=gmin*(alpha_n+qLR)/(1-h+glower+kappa*(gLR-glower).^(eta-1).*((eta-1)*gLR+glower));
b3min=-phi*qmin/gmin;
c3min=1-h+(1+r-gLR)*b3min-(gLR-glower)-kappa*(gLR-glower).^eta;
c2min=(beta*(1-h+glower+kappa*(gLR-glower).^(eta-1).*((eta-1)*gLR+glower)))^(-1/gamma)*gmin*c3min;
w2min=c2min+b3min*gmin+h+1e-5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
B_state=(w2min-A)/(1+r); %% minimum grid is defined when c approaches h, g approaches gmin and constraint binds
Bmax=-1e-5;  %% max of bond space
NB=1000;  %% # of grids
Bgrid=linspace(min(B_state), Bmax, NB)';

load baseline_result
load UE_policyfunction
load distortive_tax
cCSP_dis=cCSP;gCSP_dis=gCSP; BCSP_dis=BCSP;qCSP_dis=qCSP;taub_dis=taub;taun_dis=taun;
load investment_subsidy

i=2;
figure(1)
plot(Bgrid(Bgrid>B_state(i)),B_LF1(Bgrid>B_state(i),i),'Color',[1 0.2 0.2],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),B_CSP1(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),BCSP(Bgrid>B_state(i),i),':','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),BUSP(Bgrid>B_state(i),i),'.','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),BCSP_dis(Bgrid>B_state(i),i),'-.','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),Bgrid(Bgrid>B_state(i)),'-','Color','black');hold on
legend('CE','MP','GS','MI','OP','45 line','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('$\hat{b}_{t+1}$','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'b_policy', 'pdf')

figure(2)
plot(Bgrid(Bgrid>B_state(i)),g_LF1(Bgrid>B_state(i),i),'Color',[1 0.2 0.2],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),g_CSP1(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),gCSP(Bgrid>B_state(i),i),':','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),gUSP(Bgrid>B_state(i),i),'.','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),gCSP_dis(Bgrid>B_state(i),i),'-.','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('${g}_{t+1}$','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'g_policy', 'pdf')

figure(3)
plot(Bgrid(Bgrid>B_state(i)),C_LF1(Bgrid>B_state(i),i),'Color',[1 0.2 0.2],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),C_CSP1(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),cCSP(Bgrid>B_state(i),i),':','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),cUSP(Bgrid>B_state(i),i),'.','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),cCSP_dis(Bgrid>B_state(i),i),'-.','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('$\hat{c}^h_{t}$','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'c_policy', 'pdf')

figure(4)
plot(Bgrid(Bgrid>B_state(i)),q_LF1(Bgrid>B_state(i),i),'Color',[1 0.2 0.2],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),q_CSP1(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),qCSP(Bgrid>B_state(i),i),':','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),qUSP(Bgrid>B_state(i),i),'.','LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),qCSP_dis(Bgrid>B_state(i),i),'-.','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('$\hat{q}_{t}$','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'q_policy', 'pdf')


figure(5)
plot(Bgrid(Bgrid>B_state(i)),100*taubUSP(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),100*tauzUSP(Bgrid>B_state(i),i),':','LineWidth',2);hold on
legend('\tau_t^{MP, b}','\tau_t^z','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('MI (\%)','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'MI_tau', 'pdf')

figure(6)
plot(Bgrid(Bgrid>B_state(i)),100*taub_dis(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot(Bgrid(Bgrid>B_state(i)),100*taun_dis(Bgrid>B_state(i),i),':','LineWidth',2);hold on
legend('\tau_t^{MP, b}','\tau_t^n','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('OP (\%)','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'OP_tau', 'pdf')

figure(7)
plot(Bgrid(Bgrid>B_state(i)),100*tauz(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
legend('\tau^z_{t}','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('GS (\%)','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'GS_tau', 'pdf')

figure(8)
plot(Bgrid(Bgrid>B_state(i)),100*taub_CSP1(Bgrid>B_state(i),i),'--','Color',[0.2 0.8 0.5],'LineWidth',2);hold on
legend('\tau^{MP, b}_{t}','location','southeast')
grid on
xlabel('$\hat{b}_t$','interpret','latex')
title('MP (\%)','interpret','latex')
xlim([-0.3 -0.15])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'MP_tau', 'pdf')


%%%% simulation
T=11000;
y_index=[1:NA];
y_initial=y_index(ceil(NA/2));
y_simu=MarkovChain(y_index,Prob,T,y_initial,1);
b_initial=median(Bgrid);
for t=1:T
    if t==1
        b_state=b_initial;
    else
        b_state=b_simu(t-1,1);
    end
    c_simu(t,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),C_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    g_simu(t,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),g_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    b_simu(t,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),B_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    q_simu(t,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),q_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');

    if b_state<=b_marginalLF(y_simu(t))
        constrained_simu(t,1)=1;
    else
        constrained_simu(t,1)=0;
    end
    
    if t>1000
        if t==1001
            b_stateC=b_simu(t-1,1);
            b_stateU=b_simu(t-1,1);
            b_stateG=b_simu(t-1,1);
            b_stateD=b_simu(t-1,1);
        else
            b_stateC=b_simuC(t-1001,1);
            b_stateU=b_simuU(t-1001,1);            
            b_stateG=b_simuG(t-1001,1);
            b_stateD=b_simuD(t-1001,1);            
        end
        c_simuC(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),C_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        g_simuC(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),g_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        b_simuC(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),B_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        q_simuC(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),q_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        taub_simuC(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taub_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');

        c_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        g_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        b_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        q_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        taub_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taubUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        tauz_simuU(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),tauzUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        
        c_simuG(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        g_simuG(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        b_simuG(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        q_simuG(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        tauz_simuG(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),tauz(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');

        c_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        g_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        b_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        q_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        taub_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taub_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        taun_simuD(t-1000,1)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taun_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');

        if b_stateC<=b_marginalCSP(y_simu(t))
            constrained_simuC(t-1000,1)=1;
        else
            constrained_simuC(t-1000,1)=0;
        end

        if b_stateU<=b_marginalUSP(y_simu(t))
            constrained_simuU(t-1000,1)=1;
        else
            constrained_simuU(t-1000,1)=0;
        end
        
        if b_stateG<=b_marginal(y_simu(t))
            constrained_simuG(t-1000,1)=1;
        else
            constrained_simuG(t-1000,1)=0;
        end

        if b_stateD<=b_marginalCSP_dis(y_simu(t))
            constrained_simuD(t-1000,1)=1;
        else
            constrained_simuD(t-1000,1)=0;
        end
        
    end    
end
c_simu(1:1000)=[];y_simu(1:1000)=[];q_simu(1:1000)=[];g_simu(1:1000)=[];constrained_simu(1:1000)=[];
gdp_simu=A(y_simu);c_simu=c_simu+h;
c_simuC=c_simuC+h;
c_simuU=c_simuU+h; %% Total GDP
c_simuG=c_simuG+h; %% Total GDP
c_simuD=c_simuD+h; %% Total GDP

b0_simu=b_simu(1000:end-1);
b1_simu=b_simu(1001:end);
ca_simu=100*(b1_simu.*g_simu-b0_simu)./gdp_simu;

b0_simuC=[b_simu(1000);b_simuC(1:end-1)];
b1_simuC=b_simuC(1:end);

b0_simuU=[b_simu(1000);b_simuU(1:end-1)];
b1_simuU=b_simuU(1:end);

b0_simuG=[b_simu(1000);b_simuG(1:end-1)];
b1_simuG=b_simuG(1:end);

b0_simuD=[b_simu(1000);b_simuD(1:end-1)];
b1_simuD=b_simuD(1:end);

event=constrained_simu==1&ca_simu>mean(ca_simu)+std(ca_simu);
Event=find(event==1);
Event= Event(Event>6 & Event<length(b0_simu)-4);
clear B_Moments g_Moments G_Moments c_Moments ProbConstrained_Moments A_Moments q_Moments mu_Moments
clear B_MomentsC g_MomentsC G_MomentsC c_MomentsC ProbConstrained_MomentsC  q_MomentsC mu_MomentsC
clear B_MomentsU g_MomentsU G_MomentsU c_MomentsU ProbConstrained_MomentsU  q_MomentsU mu_MomentsU

for i=1:length(Event)
    B_Moments(i,:)=b1_simu(Event(i)-5:Event(i)+5)';  %% b_t
    g_Moments(i,:)=100*log(g_simu(Event(i)-5:Event(i)+5)'); %% Endo Growth rate
    G_Moments(i,:)=100*(log(A(y_simu(Event(i)-5:Event(i)+5))')...
        -log(A(y_simu(Event(i)-6:Event(i)+4))')+log(g_simu(Event(i)-6:Event(i)+4)'));%% total growth rate
    c_Moments(i,:)=c_simu(Event(i)-5:Event(i)+5)'; %% total consumption
    ProbConstrained_Moments(i,:)=constrained_simu(Event(i)-5:Event(i)+5)';
    A_Moments(i,:)=A(y_simu(Event(i)-5:Event(i)+5))';
    q_Moments(i,:)=q_simu(Event(i)-5:Event(i)+5)';

    B_MomentsC(i,:)=b1_simuC(Event(i)-5:Event(i)+5)';  %% b_t
    g_MomentsC(i,:)=100*log(g_simuC(Event(i)-5:Event(i)+5)'); %% Endo Growth rate
    G_MomentsC(i,:)=100*(log(A(y_simu(Event(i)-5:Event(i)+5))')...
        -log(A(y_simu(Event(i)-6:Event(i)+4))')+log(g_simuC(Event(i)-6:Event(i)+4)'));%% total growth rate
    c_MomentsC(i,:)=c_simuC(Event(i)-5:Event(i)+5)'; %% total consumption
    ProbConstrained_MomentsC(i,:)=constrained_simuC(Event(i)-5:Event(i)+5)';
    q_MomentsC(i,:)=q_simuC(Event(i)-5:Event(i)+5)';
    taub_MomentsC(i,:)=taub_simuC(Event(i)-5:Event(i)+5)';

    B_MomentsU(i,:)=b1_simuU(Event(i)-5:Event(i)+5)';  %% b_t
    g_MomentsU(i,:)=100*log(g_simuU(Event(i)-5:Event(i)+5)'); %% Endo Growth rate
    G_MomentsU(i,:)=100*(log(A(y_simu(Event(i)-5:Event(i)+5))')...
        -log(A(y_simu(Event(i)-6:Event(i)+4))')+log(g_simuU(Event(i)-6:Event(i)+4)'));%% total growth rate
    c_MomentsU(i,:)=c_simuU(Event(i)-5:Event(i)+5)'; %% total consumption
    ProbConstrained_MomentsU(i,:)=constrained_simuU(Event(i)-5:Event(i)+5)';
    q_MomentsU(i,:)=q_simuU(Event(i)-5:Event(i)+5)';
    taub_MomentsU(i,:)=taub_simuU(Event(i)-5:Event(i)+5)';
    tauz_MomentsU(i,:)=tauz_simuU(Event(i)-5:Event(i)+5)';

    B_MomentsG(i,:)=b1_simuG(Event(i)-5:Event(i)+5)';  %% b_t
    g_MomentsG(i,:)=100*log(g_simuG(Event(i)-5:Event(i)+5)'); %% Endo Growth rate
    G_MomentsG(i,:)=100*(log(A(y_simu(Event(i)-5:Event(i)+5))')...
        -log(A(y_simu(Event(i)-6:Event(i)+4))')+log(g_simuG(Event(i)-6:Event(i)+4)'));%% total growth rate
    c_MomentsG(i,:)=c_simuG(Event(i)-5:Event(i)+5)'; %% total consumption
    ProbConstrained_MomentsG(i,:)=constrained_simuG(Event(i)-5:Event(i)+5)';
    q_MomentsG(i,:)=q_simuG(Event(i)-5:Event(i)+5)';
    tauz_MomentsG(i,:)=tauz_simuG(Event(i)-5:Event(i)+5)';

    B_MomentsD(i,:)=b1_simuD(Event(i)-5:Event(i)+5)';  %% b_t
    g_MomentsD(i,:)=100*log(g_simuD(Event(i)-5:Event(i)+5)'); %% Endo Growth rate
    G_MomentsD(i,:)=100*(log(A(y_simu(Event(i)-5:Event(i)+5))')...
        -log(A(y_simu(Event(i)-6:Event(i)+4))')+log(g_simuD(Event(i)-6:Event(i)+4)'));%% total growth rate
    c_MomentsD(i,:)=c_simuD(Event(i)-5:Event(i)+5)'; %% total consumption
    ProbConstrained_MomentsD(i,:)=constrained_simuD(Event(i)-5:Event(i)+5)';
    q_MomentsD(i,:)=q_simuD(Event(i)-5:Event(i)+5)';
    taub_MomentsD(i,:)=taub_simuD(Event(i)-5:Event(i)+5)';
    taun_MomentsD(i,:)=taun_simuD(Event(i)-5:Event(i)+5)';

end

figure(9)
plot([-5:5],mean(A_Moments),'-','LineWidth',2,'color',[1 0.2 0.2]); 
axis tight
grid on
title('$\theta_t$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFA_main_all', 'pdf')


figure(10)
plot([-5:5],mean(c_Moments),'LineWidth',2,'color',[1 0.2 0.2]); hold on
plot([-5:5],mean(c_MomentsC),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],mean(c_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(c_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(c_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
axis tight
grid on
title('$\hat{c}_t$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFc_main_all', 'pdf')

figure(11)
plot([-5:5],mean(B_Moments),'LineWidth',2,'color',[1 0.2 0.2]); hold on
plot([-5:5],mean(B_MomentsC),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],mean(B_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(B_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(B_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
title('$\hat{b}_{t+1}$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFb_main_all', 'pdf')

figure(12)
plot([-5:5],mean(G_Moments),'LineWidth',2,'color',[1 0.2 0.2]); hold on
plot([-5:5],mean(G_MomentsC),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],mean(G_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(G_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(G_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
axis tight
grid on
title('$100 \log \frac{y_{t}}{y_{t-1}}$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFgg_main_all', 'pdf')

figure(13)
plot([-5:5],mean(g_Moments),'LineWidth',2,'color',[1 0.2 0.2]); hold on
plot([-5:5],mean(g_MomentsC),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],mean(g_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(g_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(g_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
axis tight
grid on
title('$100 \log g_{t+1}$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFg_main_all', 'pdf')

figure(14)
plot([-5:5],mean(q_Moments),'LineWidth',2,'color',[1 0.2 0.2]); hold on
plot([-5:5],mean(q_MomentsC),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],mean(q_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(q_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(q_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
axis tight
grid on
title('$\hat{q}_{t}$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFq_main_all', 'pdf')
% 
figure(15)
plot([-5:5],mean(ProbConstrained_Moments),'-','Color',[1 0.2 0.2],'LineWidth',2); hold on
plot([-5:5],mean(ProbConstrained_MomentsC),'--','LineWidth',2); hold on
plot([-5:5],mean(ProbConstrained_MomentsG),'-','LineWidth',2); hold on
plot([-5:5],mean(ProbConstrained_MomentsU),':','LineWidth',2); hold on
plot([-5:5],mean(ProbConstrained_MomentsD),'-o','LineWidth',2); hold on
legend('CE','MP','GS','MI','OP','location','southeast')
% legend('CE','MP','location','northeast')
axis tight
grid on
title('$1_{\hat{\mu}_{t}>0}$','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFConstrained_main_all', 'pdf')

figure(16)
plot([-5:5],100*mean(taub_MomentsC),'-','Color',[1 0.2 0.2],'LineWidth',2); hold on
plot([-5:5],100*mean(taub_MomentsU),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
plot([-5:5],100*mean(taub_MomentsD),':','LineWidth',2); hold on
legend('MP','MI','OP','location','northeast')
axis tight
grid on
title('$\tau^{MP, b}_{t}$(\%)','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFtaub_main_all', 'pdf')

figure(17)
plot([-5:5],100*mean(tauz_MomentsU),'-','Color',[1 0.2 0.2],'LineWidth',2); hold on
plot([-5:5],100*mean(tauz_MomentsG),'--','Color',[0.2 0.8 0.5],'LineWidth',2); hold on
legend('MI','GS','location','southeast')
axis tight
grid on
title('$\tau^z_{t}$ (\%)','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFtauz_main_all', 'pdf')

figure(18)
plot([-5:5],100*mean(taun_MomentsD),'-','Color',[1 0.2 0.2],'LineWidth',2); hold on
legend('OP','location','southeast')
axis tight
grid on
title('$\tau^n_{t}$ (\%)','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'IRFtaun_main_all', 'pdf')




%%%% long run and short run dynamics
clear c_simu g_simu b_simu q_simu 
clear c_simuC g_simuC b_simuC q_simuC 
clear c_simuU g_simuU b_simuU q_simuU 
clear c_simuG g_simuG b_simuG q_simuG 
clear c_simuD g_simuD b_simuD q_simuD 
clear taun_simuD taub_simuD tauz_simuG taub_simuU tauz_simuU taub_simuC
clear constrained_simu constrained_simuC constrained_simuU constrained_simuG constrained_simuD
T=1100;
N=1000;
y_index=[1:NA];
y_initial=y_index(ceil(NA/2));
b_initial=median(Bgrid);
g_simu=ones(T,N);
c_simu=ones(T,N);
b_simu=ones(T,N);
q_simu=ones(T,N);
mu_simu=ones(T,N);
constrained_simu=ones(T,N);
yy_simu=ones(T-1000,N);
for n=1:N
    y_simu=MarkovChain(y_index,Prob,T,y_initial,n);
for t=1:T
    if t==1
        b_state=b_initial;
    else
        b_state=b_simu(t-1,n);
    end
    c_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),C_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    g_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),g_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    b_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),B_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
    q_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),q_LF1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');

    if b_state<=b_marginalLF(y_simu(t))
        constrained_simu(t,n)=1;
    else
        constrained_simu(t,n)=0;
    end
    
    if t>1000
        if t==1001
            b_stateC=b_simu(t-1,n);
            b_stateU=b_simu(t-1,n);
            b_stateG=b_simu(t-1,n);
            b_stateD=b_simu(t-1,n);
        else
            b_stateC=b_simuC(t-1001,n);
            b_stateU=b_simuU(t-1001,n);            
            b_stateG=b_simuG(t-1001,n);
            b_stateD=b_simuD(t-1001,n);            
        end
        c_simuC(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),C_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        g_simuC(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),g_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        b_simuC(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),B_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        q_simuC(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),q_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');
        taub_simuC(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taub_CSP1(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateC,'linear','extrap');

        c_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        g_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        b_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        q_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        taub_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taubUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        tauz_simuU(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),tauzUSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateU,'linear','extrap');
        
        c_simuG(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        g_simuG(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        b_simuG(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        q_simuG(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qCSP(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');
        tauz_simuG(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),tauz(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateG,'linear','extrap');

        c_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        g_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        b_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        q_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),qCSP_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        taub_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taub_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');
        taun_simuD(t-1000,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),taun_dis(Bgrid>B_state(y_simu(t)),y_simu(t)),b_stateD,'linear','extrap');

        if b_stateC<=b_marginalCSP(y_simu(t))
            constrained_simuC(t-1000,n)=1;
        else
            constrained_simuC(t-1000,n)=0;
        end

        if b_stateU<=b_marginalUSP(y_simu(t))
            constrained_simuU(t-1000,n)=1;
        else
            constrained_simuU(t-1000,n)=0;
        end
        
        if b_stateG<=b_marginal(y_simu(t))
            constrained_simuG(t-1000,n)=1;
        else
            constrained_simuG(t-1000,n)=0;
        end

        if b_stateD<=b_marginalCSP_dis(y_simu(t))
            constrained_simuD(t-1000,n)=1;
        else
            constrained_simuD(t-1000,n)=0;
        end
        
    end    
end
        yy_simu(:,n)=y_simu(1001:end);
        n
end

BB_initial=b_simu(1000,:);
YY_initial=yy_simu(1,:);
g_simu(1:1000,:)=[];
c_simu(1:1000,:)=[];
b_simu(1:1000,:)=[];
constrained_simu(1:1000,:)=[];
q_simu(1:1000,:)=[];
mu_simu(1:1000,:)=[];

figure(19)
plot([1:T-1000],mean(g_simu,2),'color',[1 0.2 0.2],'LineWidth',2);hold on
plot([1:T-1000],mean(g_simuC,2),':','color',[0 0.5 .8],'LineWidth',2);hold on
plot([1:T-1000],mean(g_simuG,2),'--','color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot([1:T-1000],mean(g_simuU,2),'-.','LineWidth',2);hold on
plot([1:T-1000],mean(g_simuD,2),'-o','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP')
xlim([1 20])
axis tight
xlim([1,20])
title('$g_{t+1}$','interpret','latex')
grid on
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'Mcg_main_all', 'pdf')


figure(20)
plot([1:T-1000],mean(b_simu,2),'color',[1 0.2 0.2],'LineWidth',2);hold on
plot([1:T-1000],mean(b_simuC,2),':','color',[0 0.5 .8],'LineWidth',2);hold on
plot([1:T-1000],mean(b_simuG,2),'--','color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot([1:T-1000],mean(b_simuU,2),'-.','LineWidth',2);hold on
plot([1:T-1000],mean(b_simuD,2),'-o','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP')
title('$\hat{b}_{t+1}$','interpret','latex')
axis tight
grid on
xlim([1,20])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'Mcb_main_all', 'pdf')




figure(21)
plot([1:T-1000],mean(c_simu,2),'color',[1 0.2 0.2],'LineWidth',2);hold on
plot([1:T-1000],mean(c_simuC,2),':','color',[0 0.5 .8],'LineWidth',2);hold on
plot([1:T-1000],mean(c_simuG,2),'--','color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot([1:T-1000],mean(c_simuU,2),'-.','LineWidth',2);hold on
plot([1:T-1000],mean(c_simuD,2),'-o','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP')
title('$\hat{c}^h_t$','interpret','latex')
axis tight
xlim([1,20])
grid on
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'Mcc_main_all', 'pdf')


figure(22)
plot([1:T-1000],mean(q_simu,2),'color',[1 0.2 0.2],'LineWidth',2);hold on
plot([1:T-1000],mean(q_simuC,2),':','color',[0 0.5 .8],'LineWidth',2);hold on
plot([1:T-1000],mean(q_simuG,2),'--','color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot([1:T-1000],mean(q_simuU,2),'-.','LineWidth',2);hold on
plot([1:T-1000],mean(q_simuD,2),'-o','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP')
title('$\hat{q}_t$','interpret','latex')
axis tight
grid on
xlim([1,20])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'Mcq_main_all', 'pdf')


figure(23)
plot([1:T-1000],mean(constrained_simu,2),'color',[1 0.2 0.2],'LineWidth',2);hold on
plot([1:T-1000],mean(constrained_simuC,2),':','color',[0 0.5 .8],'LineWidth',2);hold on
plot([1:T-1000],mean(constrained_simuG,2),'--','color',[0.2 0.8 0.5],'LineWidth',2);hold on
plot([1:T-1000],mean(constrained_simuU,2),'-.','LineWidth',2);hold on
plot([1:T-1000],mean(constrained_simuD,2),'-o','LineWidth',2);hold on
legend('CE','MP','GS','MI','OP')
title('$1_{\hat{\mu}_{t}>0}$','interpret','latex')
axis tight
xlim([1,20])
grid on
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'Mcconstrained_main_all', 'pdf')



figure(24)
plot([1:T-1000],mean(A(yy_simu),2),'color','black','LineWidth',2);hold on
title('$\theta_t$','interpret','latex')
axis tight
grid on
xlim([1,20])
set(gca,'FontSize', 20)
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
% saveas(gcf, 'McA_main_all', 'pdf')

