%% construct event windows
clear;close all;clc
%% use Calvo's episodes
[xlsdata, xlstext] =xlsread('Calibration_Data.xlsx','Calvo_episodes');
calvo_time=xlsdata(:,4);
calvo_country=xlsdata(:,3);

%% use KM's episodes
[xlsdata, xlstext] =xlsread('Calibration_Data.xlsx','Korinek_Mendoza');
KM_time=xlsdata(:,5);
KM_country=xlsdata(:,3);

%% use GDP and TFP 
[xlsdata, xlstext] =xlsread('Calibration_Data.xlsx','GDP per capita growth');
gdp_time=[1961:2015];
gdp_pc_data=xlsdata(1:55,1:55);
[xlsdata, xlstext] =xlsread('Calibration_Data.xlsx','TFP_Penn');
TFP_data=xlsdata(2:56,2:56);

%% construct event study of Calvo episode
Calvo_N=33;
Calvo_growth_pc=nan(Calvo_N,11); %% construct 11 years window
Calvo_TFP=nan(Calvo_N,11); %% construct 11 years window
 for i=1:Calvo_N
    time=find(gdp_time==calvo_time(i));
    Calvo_growth_pc(i,:)=gdp_pc_data(time-5:time+5,calvo_country(i));
    Calvo_TFP(i,:)=diff(log(TFP_data(time-6:time+5,calvo_country(i))))*100;
end
figure(1)
plot([-5:5],mean(Calvo_growth_pc),'-o','color','red','linewidth',2);hold on
plot([-5:5],median(Calvo_growth_pc),'-o','color','blue','linewidth',2);
legend('Mean','Median','location','southeast')
axis tight
grid on
title('Output','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'GDP_Calvo', 'pdf')



figure(2)
plot([-5:5],nanmean(Calvo_TFP),'-o','color','red','linewidth',3);hold on
plot([-5:5],nanmedian(Calvo_TFP),'-o','color','blue','linewidth',3);
legend('Mean','Median','location','southeast')
grid on
axis tight
title('TFP','interpret','latex')
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'TFP_Calvo', 'pdf')


%% construct event study of KM episodes
KM_N=137;
KM_growth_pc=nan(KM_N,11);
KM_TFP=nan(KM_N,11);

for i=1:KM_N
    time=find(gdp_time==KM_time(i));
    if time+5>55 
        KM_growth_pc(i,1:61-time)=gdp_pc_data(time-5:55,KM_country(i));
        KM_TFP(i,1:61-time)=diff(log(TFP_data(time-6:55,KM_country(i))))*100; 
    else
        KM_growth_pc(i,:)=gdp_pc_data(time-5:time+5,KM_country(i));
        KM_TFP(i,:)=diff(log(TFP_data(time-6:time+5,KM_country(i))))*100;
    end
end

figure(3)
plot([-5:5],nanmean(KM_growth_pc),'-o','color','red','linewidth',2);hold on
plot([-5:5],nanmedian(KM_growth_pc),'-o','color','blue','linewidth',2);
legend('Mean','Median','location','southeast')
title('Output','interpret','latex')
axis tight
grid on
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'GDP_KM', 'pdf')



figure(4)
plot([-5:5],nanmean(KM_TFP),'-o','color','red','linewidth',3);hold on
plot([-5:5],nanmedian(KM_TFP),'-o','color','blue','linewidth',3);
legend('Mean','Median','location','southeast')
title('TFP','interpret','latex')
axis tight
grid on
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'TFP_KM', 'pdf')




%% construct counter-factual
figure(5)
T=11;
ygrowth=mean(Calvo_growth_pc);
GDP_counter=100*ones(2,T);
for t=2:T
    GDP_counter(1,t)=GDP_counter(1,t-1)*exp(ygrowth(t)/100); %%% real data
    GDP_counter(2,t)=GDP_counter(2,t-1)*exp(mean(ygrowth(1:5))/100); %%% counter-factual path
end
plot([-5:5],GDP_counter(1,:),'Color','blue','LineStyle','-','LineWidth',3);hold on
plot([-5:5],GDP_counter(2,:),'Color','red','LineStyle','--','LineWidth',3);
title('Sudden stop episodes','interpret','latex')
grid on
axis tight
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'output_dynamics', 'pdf')


YY=100*ones(3,11);
YG=[0.02 0.02 0.02 0.02 0.02 -0.06 0.02 0.02 0.02 0.02 0.02;
    0.02 0.02 0.02 0.02 0.02 -0.06 0.1 0.02 0.02 0.02 0.02    
    0.02 0.02 0.02 0.02 0.02 0.02 0.02 0.02 0.02 0.02 0.02];

for t=2:11
    YY(:,t)=YY(:,t-1).*exp(YG(:,t));
end
    
figure(6)
plot([-5:5],YY(1,:),'Color','blue','LineStyle','-','LineWidth',3);hold on
plot([-5:5],YY(2,:),'Color','black','LineStyle',':','LineWidth',3);hold on
plot([-5:5],YY(3,:),'Color','red','LineStyle','--','LineWidth',3);hold on
title('Output effects of crises','interpret','latex')
legend('Data','Existing Models','location','northwest')
grid on
axis tight
set(gcf, 'PaperPosition', [0 0 5 5]); %Position the plot further to the left and down. Extend the plot to fill entire paper.
set(gcf, 'PaperSize', [5 5]); %Keep the same paper size
set(gca,'FontSize', 20)
% saveas(gcf, 'output_behavior', 'pdf')   

    