%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%           Model  with a quardratic function of growth function:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;close all; clear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   1. Set Parameters

global r gamma alpha_n  eta beta kappa glower h phi rho sigma

%%%%%%%%%%%%%%%%%% Parameters taken from literature

r          = .06;                     %% risk-free interest rate
gamma = 2;                       %% risk aversion parameter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% Auxiliary Variables

gLR=1.023;                        %% long run growth
gupper=1.05;                     %% highest growth rate: arbitrarily set

%%%%%%%%%%%%%%%%%%% Parameters set to match moments

beta=0.968;                       %% discount rate
glower=0.95;                     %%  parameter in Psi function
eta=2;                               %%  quardratic functional form

investment=1-0.3*(1+r-gLR)/gLR-.776;                            %% investment is implied by the budget constraint in steady states
i_aux=investment-gLR+glower;                                         %% i_aux is the quardratic term in the Psi function
h=1+glower+i_aux/(gLR-glower)*((eta-1)*gLR+glower)...
    -(1+eta/(gLR-glower)*i_aux)/beta*gLR^gamma;             %% habit parameter, chosen to match Euler Equation for z
kappa=i_aux/(gLR-glower)^eta;                                         %% Average Growth Enhancing Expenditure

bLR=-0.3/gLR;                                                                  %% NFA=-30% of GDP
alpha_n=0.2;                                                                     %% Share of Asset Income
qLR=beta*gLR^(1-gamma)/(1-beta*gLR^(1-gamma))*alpha_n; %% Asset price in steady state
phi=(-bLR*gLR)/qLR;                                                             %% Collateral Constraint Parameter


%%%%%%%%%%%%%%%%%  Exogenous TFP process %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rho=.83;                             %% persistence parameter
sigma=0.04;                       %% volatility parameter


NA=5;                    %% # of shocks space
[logA, Prob]=rouwenhorst(rho,sigma,NA);  %% use Rouwenhorst's method to discretize

A=exp(logA)'; %% realization of shocks
state_Prob=Prob^1000; state_Prob=state_Prob(1,:);%% Long run Probability
EA=state_Prob*A; %% long run technology

%%%%%%%%%%%%%%%%%%%%%%%%%% Create State Space
%% Find the minimum level of bond in the state space %%%%%%%%%%%
gmin=glower;
qmin=gmin*(alpha_n+qLR)/(1-h+glower+kappa*(gLR-glower).^(eta-1).*((eta-1)*gLR+glower));
b3min=-phi*qmin/gmin;
c3min=1-h+(1+r-gLR)*b3min-(gLR-glower)-kappa*(gLR-glower).^eta;
c2min=(beta*(1-h+glower+kappa*(gLR-glower).^(eta-1).*((eta-1)*gLR+glower)))^(-1/gamma)*gmin*c3min;
w2min=c2min+b3min*gmin+h+1e-5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Bbar=(min(A)-h)/(1+r-gmin);
wmin=h-Bbar*gmin;
B_state=(wmin-A)/(1+r);
Bbar=-max(B_state);
% B_state=-3.72;
Bmax=-1e-5;  %% max of bond space
NB=1000;  %% # of grids
Bgrid=linspace(min(B_state), Bmax, NB)';
BgridUE=Bgrid;
B_stateUE=B_state;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('UE_policyfunction')
T=11000;
y_index=[1:NA];
y_initial=y_index(ceil(NA/2));
y_simu=MarkovChain(y_index,Prob,T,y_initial,1);
b_initial=median(Bgrid);  %% start from a median of Bgrid
for t=1:T
    if t==1
        b_state=b_initial;
    else
        b_state=b_simu(t-1,1);
    end
    c_simu(t,1)=interp1(BgridUE,cUE(:,y_simu(t)),b_state,'linear','extrap');
    g_simu(t,1)=interp1(BgridUE,gUE(:,y_simu(t)),b_state,'linear','extrap');
    b_simu(t,1)=interp1(BgridUE,BUE(:,y_simu(t)),b_state,'linear','extrap');
    if b_simu(t,1)==-Bbar
        constrained_simu(t,1)=1;
    else
        constrained_simu(t,1)=0;
    end
end
c_simu(1:1000)=[];y_simu(1:1000)=[];g_simu(1:1000)=[];constrained_simu(1:1000)=[];
gdp_simu=A(y_simu); c_simu=c_simu+h;%% Total GDP and consumption
b0_simu=b_simu(1000:end-1);
b1_simu=b_simu(1001:end);
ca_simu=100*(b1_simu.*g_simu-b0_simu)./gdp_simu;
nfa_simu=100*b1_simu.*g_simu./gdp_simu;

m_simu=ones(size(c_simu)); %% trend component
for i=2:length(m_simu)
    m_simu(i)=m_simu(i-1)*g_simu(i-1);
end

C_simu=c_simu.*m_simu;  %%% Orginal Consumption
GDP_simu=gdp_simu.*m_simu; %% Original GDP
[YT, YC]=hpfilter(log(GDP_simu),100);
[CT, CC]=hpfilter(log(C_simu),100);
YC=100*YC;
CC=100*CC;


display('Mean: NFA, Prob of crisis')
[mean(nfa_simu)  100*mean(constrained_simu==1&ca_simu>mean(ca_simu)+std(ca_simu))]
display('Growth rate: Mean')
 [((GDP_simu(end)/GDP_simu(1))^(1/length(GDP_simu))-1)*100]
display('Current account corr with GDP')
[corr2(ca_simu,YC)]
display('Composition: c Psi')
[mean(100*c_simu./gdp_simu)]


load benchmarkmodel
N=1000; %% # of simulation
T=1100;   %% 100 years 
y_index=[1:NA];
y_initial=y_index(ceil(NA/2));
b_initial=median(Bgrid);
g_simu=ones(T,N);
c_simu=ones(T,N);
b_simu=ones(T,N);
q_simu=ones(T,N);
yy_simu=ones(T-1000,N);
B_state=(w2min-A)/(1+r); %% minimum grid is defined when c approaches h, g approaches gmin and constraint binds

for n=1:N
    y_simu=MarkovChain(y_index,Prob,T,y_initial,n);
    for t=1:T
        if t==1
            b_state=b_initial;
        else
            b_state=b_simu(t-1,n);
        end
        c_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),cLF(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
        g_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),gLF(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');
        b_simu(t,n)=interp1(Bgrid(Bgrid>B_state(y_simu(t))),BLF(Bgrid>B_state(y_simu(t)),y_simu(t)),b_state,'linear','extrap');

        if t>1000
            if t==1001
                b_stateCSP=b_simu(t-1,n);              
            else
                b_stateCSP=b_simu_controlC(t-1001,n);
            end
        c_simu_controlC(t-1000,n)=interp1(BgridUE(BgridUE>B_stateUE(y_simu(t))),cUE(BgridUE>B_stateUE(y_simu(t)),y_simu(t)),b_stateCSP,'linear','extrap');
        g_simu_controlC(t-1000,n)=interp1(BgridUE(BgridUE>B_stateUE(y_simu(t))),gUE(BgridUE>B_stateUE(y_simu(t)),y_simu(t)),b_stateCSP,'linear','extrap');
        b_simu_controlC(t-1000,n)=interp1(BgridUE(BgridUE>B_stateUE(y_simu(t))),BUE(BgridUE>B_stateUE(y_simu(t)),y_simu(t)),b_stateCSP,'linear','extrap');
        end
        yy_simu(:,n)=y_simu(1001:end);
    end
end
BB_initial=b_simu(1000,:);
YY_initial=yy_simu(1,:);
g_simu(1:1000,:)=[];
c_simu(1:1000,:)=[];
b_simu(1:1000,:)=[];
constrained_simu(1:1000,:)=[];


M1=ones(T-1000,N);
M2=ones(T-1000,N);
for n=1:N
    for t=2:T-1000
        M1(t,n)=M1(t-1,n)*g_simu(t-1,n);
        M2(t,n)=M2(t-1,n)*g_simu_controlC(t-1,n);
    end
end
C1=c_simu.*M1;
C2=c_simu_controlC.*M2;

V1=sum(kron(beta.^[0:T-1000-1]',ones(1,N)).*C1.^(1-gamma)/(1-gamma));
V2=sum(kron(beta.^[0:T-1000-1]',ones(1,N)).*C2.^(1-gamma)/(1-gamma));
%% control for consumption
V1_counter1=sum(kron(beta.^[0:T-1000-1]',ones(1,N)).*(c_simu.*M2).^(1-gamma)/(1-gamma));
%% control for growth
V1_counter2=sum(kron(beta.^[0:T-1000-1]',ones(1,N)).*(c_simu_controlC.*M1).^(1-gamma)/(1-gamma));

%% Welfare Decomposition
display ('Welfare: Overall, Trend, Cyclical')
VCC_deco=[mean((V2./V1).^(1-gamma)-1)*100 mean((V1_counter1./V1).^(1-gamma)-1)*100 mean((V1_counter2./V1).^(1-gamma)-1)*100]

